/*
 * Copyright 2005 Sun Microsystems, Inc.
 * Property of Sun Microsystems, Inc.
 */


package com.sun.im.provider.sample;

import org.netbeans.lib.collab.util.StringUtility;
import com.iplanet.im.server.util.WildCardPattern;

import java.util.*;
import java.io.*;
import com.sun.im.service.*;
import com.sun.im.provider.*;

/**
 * Get list of users and groups from plain text files
 * similar to a unix password file
 * 
 */
public class SampleRealm implements com.sun.im.provider.Realm 
{

    class CollaborationPrincipalImpl implements CollaborationPrincipal {
        
        String _uid;
        String _domain;
        String _displayName;
        CollaborationPrincipalImpl(String uid, String domain, String displayname) {
            _uid = uid;
            _domain = domain;
            _displayName = displayname;
        }
        
        public String getUID() {
            return _uid;
        }

        public String getDisplayName() {
            return _displayName;
        }

        public String getDomainName() {
            return _domain;
        }

        public String getProperty(String attribute) {
            return null;
        }

        public void setProperty(String attribute, String value) {
        }

        public Enumeration propertyNames() {
            return null;
        }

        public void setAttributeValues(String attribute, Set values) {
        }

        public Set getAttributeValues(String attribute) {
            return null;
        }

        public String getName() {
            return _uid;
        }
      
    }
    
    class CollaborationGroupImpl extends CollaborationPrincipalImpl implements CollaborationGroup {
        private java.util.List _list = new java.util.ArrayList();
        public CollaborationGroupImpl(String uid, String domain, String displayname) {
            super(uid, domain, displayname);
        }
        
        public boolean hasMember(CollaborationPrincipal cp) {
            return _list.contains(cp);
        }
        
        public CollaborationPrincipal[] getMembers() {
            return (CollaborationPrincipal[])_list.toArray();
        }
        
        public void addMember(CollaborationPrincipal cp) {
            _list.add(cp);
        }
        
        public void removeMember(CollaborationPrincipal cp) {
            _list.remove(cp);
        }
    }
    
    class RealmSearchResultImpl implements com.sun.im.provider.RealmSearchResults {
        
        private CollaborationPrincipal[] principals = null;
        
        private int searchStatus = 0;
        
        public RealmSearchResultImpl(int searchStatus, CollaborationPrincipal[] principals) {
            this.principals = principals;
            this.searchStatus = searchStatus;
        }
        
        public CollaborationPrincipal[] getPrincipals() {
            return principals;
        }
        
        public CollaborationPrincipal[] get() {
            return principals;
        }
        
        public int getSearchStatus() {
            return searchStatus;
        }
        
        public void merge(com.sun.im.provider.RealmSearchResults rsr) {
            CollaborationPrincipal[] mergedPrincipals = (CollaborationPrincipal[])rsr.get();
            CollaborationPrincipal[] newPrincipals = new CollaborationPrincipal[getPrincipals().length + mergedPrincipals.length];
            int i = 0;
            for (i = 0; i < principals.length; i++) {
                newPrincipals[i] = principals[i];
            }
            for (int j = 0; j < mergedPrincipals.length; j++) {
                newPrincipals[i++] = (CollaborationPrincipal)mergedPrincipals[j];
            }
            principals = newPrincipals;
        }
    }
    
    final static String PASSWORD_PROP = "userPassword";
        
    /**
     * file name containing users
     */
    private final static String userFileName = "passwd";

    /**
     * file name containing groups
     */
    private final static String groupFileName = "group";
    
    /**
     * list of all groups
     */
    private Hashtable groupHT = null;

    /**
     * list of all users
     */
    private Hashtable userHT = null;    
    
    /**
     * list of all users
     */
    private Hashtable passwordHT = null;    

    /**
     * last time user file was modified
     */
    long um; 

    /**
     * last time group file was modified
     */
    long gm; 

    private String _serverDomain = null;
    
    private String _confDir = "/opt/SUNWiim/config";
    
    /**
     * create a password file realm
     * 
     * @exception com.iplanet.im.server.RealmException
     * @exception java.io.FileNotFoundException
     * @exception java.io.IOException
     */
    public SampleRealm()
    {
        ConfigReader cr = ServerUtility.getInstance().getConfigReader();
        _serverDomain = cr.getConfigValue("iim_server.domainname","");
    }
    
    /**
     * stop the realm
     */
    public void stop() {
    }
        
    /**
     * reread in the password files
     */
    public void resync() {
        try {
            passwordHT = new Hashtable();
            userHT = new Hashtable();
            groupHT = new Hashtable();
            readIn();        
        } catch (Exception e) {
            System.err.println("[SampleRealm] Reload password files failed " + e);
        }
    }
    

    /**
     * get the CollaborationGroupImpl which contains all the members of a CollaborationGroupImpl
     * 
     * @param nlg
     * @exception com.iplanet.im.server.RealmException
     */
    public CollaborationPrincipal[] expandGroup(CollaborationPrincipal principal,
                                CollaborationGroup nlg) throws RealmException 
    {
        CollaborationGroupImpl g = getGroup(nlg.getName());
        if (g != null) return (CollaborationPrincipal[])g.getMembers();
        throw new RealmException("[SampleRealm] " + nlg.getUID() + " not found.");
    }

    /**
     * get the CollaborationGroupImpl from a dn
     * 
     * @param uid unique uid for a user
     * @exception com.iplanet.im.server.RealmException
     */
    public CollaborationGroup getGroup(CollaborationPrincipal principal, String uid) throws RealmException 
    {
        return getGroup(uid);
    }

    /**
     * get the CollaborationPrincipalImpl from a uid
     * 
     * @param uid unique uid for a user
     * @exception com.iplanet.im.server.RealmException
     */
    public CollaborationPrincipal getPrincipal(CollaborationPrincipal principal, String uid) throws RealmException 
    {
        return getPrincipal(principal.getDomainName(), uid);
    }
   
   public CollaborationPrincipal getPrincipal(String searchBase, String uid) throws RealmException 
    {
        return getUser(StringUtility.getLocalPartFromAddress(uid));
    }

    /**
     * authenticate a user 
     * 
     * @param name unique id for the user
     * @param possiblePass the password the user hopes to get in with
     */

    public CollaborationPrincipal authenticate(String name, String possiblePass)  throws RealmException {

        try {             
            CollaborationPrincipalImpl u = getUser(name);            
            if (u == null) throw new RealmException("[SampleRealm] " + name + " not found.");
            String realPass = getPassword(name);
            if (realPass.equals(possiblePass)) {
                System.out.println("[SampleRealm] authenticated " + name + " using password file");
                u.setProperty(PASSWORD_PROP, realPass);
                return u;
            } else {
                System.out.println("[SampleRealm] authentication failed for " + name);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println("[SampleRealm] error while authenticating " + name + " : " + e.toString());
        }
        return null;
    }

    private List _searchPrincipals(String pattern,
                                   Map principals, 
                                   boolean byid) 
    {    
        LinkedList found = new LinkedList();
        WildCardPattern wcp = new WildCardPattern(pattern);
        for (Iterator i = principals.values().iterator(); i.hasNext(); ) {
            CollaborationPrincipalImpl p = (CollaborationPrincipalImpl)i.next();
            if (!byid) {
                if (wcp.matchIgnoreCase(p.getDisplayName())) {
                    found.add(p); 
                }
            } else {
                if (wcp.matchIgnoreCase(p.getName())) {
                    found.add(p); 
                }
            }     
        }        
        return found;
    }

    /**
     * search for users by id or name or both
     */
    public RealmSearchResults search(CollaborationPrincipal principal, String pattern, int attributeType, String filter) 
    {
        boolean byid = (attributeType == Realm.UID_ATTRIBUTE);
        return search(pattern, byid);
    }

    /**
     * search for users by id or name or both
     */
    public RealmSearchResults search(String base, String pattern,
                                     int attributeType, String filter) 
    {
        boolean byid = (attributeType == Realm.UID_ATTRIBUTE);
        return search(pattern, byid);
    }

    public RealmSearchResults search(String filter, String base) 
    {
           return search(filter, false);
    }

    /**
     * search for users by id or name or both
     */
    private RealmSearchResults search(String pattern, boolean byid)
    {                
        if (userHT == null) resync();
        List v = _searchPrincipals(pattern, userHT, byid);
        v.addAll(_searchPrincipals(pattern, groupHT, byid));
        
        CollaborationPrincipalImpl p[] = new CollaborationPrincipalImpl[v.size()]; int i = 0;
        for (Iterator iter = v.iterator(); iter.hasNext(); ) {
            p[i++] = (CollaborationPrincipalImpl)iter.next();
        }
        return new RealmSearchResultImpl(PersonalStoreSession.SEARCH_OK, p);
    }
     
    
    /**
     * read in the text files to load up the users and groups
     * 
     * @exception com.iplanet.im.server.RealmException
     * @exception java.io.FileNotFoundException
     * @exception java.io.IOException
     */
    private void readIn() throws RealmException, FileNotFoundException, IOException
    {
        //if (mod) return; //user is moding interactive so dont reload
        
        //read in all of the groups                        
        File uf = new File(_confDir, userFileName);
	if (!uf.exists()) {
	    // create empty file
	    uf.createNewFile();
	}
        File gf = new File(_confDir, groupFileName);
	if (!gf.exists()) {
	    // create empty file
	    gf.createNewFile();
	}
        long newum = uf.lastModified();
        long newgm = gf.lastModified();                    
        if (newum > um || newgm > gm) { //one was moded            
            um = newum;
            gm = newgm;
        } else {
            return;
        }
        userHT.clear();
        groupHT.clear();
        BufferedReader d = new BufferedReader(new InputStreamReader(new FileInputStream(gf), "UTF-8"));
        String line;
        while ((line = d.readLine()) != null) {
            line = line.trim();
            if (line.equals("")) continue;
            String uid, displayname;
            StringTokenizer st = new StringTokenizer(line,":");
            if (st.hasMoreTokens()) {
                uid = st.nextToken(); 
                uid = uid.trim();
            } else {
                throw new RealmException("[SampleRealm] Bad format: " + line);
            }
            int n = uid.indexOf('@');
            if (n != -1) {
                throw new RealmException("@ is not allowed in group uid " + uid);
            }
            if (st.hasMoreTokens()) {
                displayname = st.nextToken(); 
                displayname = displayname.trim();
            } else {
                throw new RealmException("[SampleRealm] Bad format: " + line);
            }
            // SCHEMA2 change  
            // will have to figure out what to pass in as orgDN/searchBase
            
            CollaborationGroupImpl g = new CollaborationGroupImpl(uid, _serverDomain, displayname);
            groupHT.put(uid, g);
        }
        d.close();
        
        //read in all of the users
        d = new BufferedReader(new InputStreamReader(new FileInputStream(uf), "UTF-8"));
        while ((line = d.readLine()) != null) {
            line = line.trim();
            if (line.equals("")) continue;                
            String user, pass, displayname, g;
            int lineIndex = 0;
            int findIndex = line.indexOf(":", lineIndex);
            if (findIndex != -1) {                    
                user = line.substring(lineIndex, findIndex);
                lineIndex = findIndex + 1;
                user = user.trim();
            } else {
                throw new RealmException("[SampleRealm] Bad format: " + line);
            }
            int n = user.indexOf('@');
            if (n != -1) {
                throw new RealmException("@ is not allowed in user uid " + user);
            }                
            findIndex = line.indexOf(":",lineIndex);
            if (findIndex != -1) {                    
                pass = line.substring(lineIndex,findIndex);
                lineIndex = findIndex + 1;                    
                pass = pass.trim();
            } else {
                throw new RealmException("[SampleRealm] Bad format: " + line);
            }
            findIndex = line.indexOf(":",lineIndex);
            if (findIndex != -1) {                    
                displayname = line.substring(lineIndex,findIndex);
                lineIndex = findIndex + 1;                    
                displayname = displayname.trim();
            } else {
                throw new RealmException("[SampleRealm] Bad format: " + line);                    
            }
            
            //now get groups string
            g = line.substring(lineIndex,line.length());
            g = g.trim();
                        
            //create the new user
            CollaborationPrincipalImpl nlu = new CollaborationPrincipalImpl(user, _serverDomain, displayname);
            
            //add them to all of the groups
            StringTokenizer gst = new StringTokenizer(g,",");
            while (gst.hasMoreElements()) {
                String groupid = gst.nextToken();
                groupid = groupid.trim();
                CollaborationGroupImpl ng = getGroup(groupid); 
                if (ng != null) ng.addMember(nlu);
            }
            
            //add the password
            passwordHT.put(user, pass);
            userHT.put(user, nlu);
            
            System.out.println("Loaded user from password files: " + nlu.getUID());
            
        }
    }
    
    /**
     * save out to the text files all of the user and the groups
     */
    public void saveOut() {                
        try {
            //save the group file
            File f = new File(_confDir, groupFileName);            
            PrintWriter out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(f), "UTF-8")));
            for (Enumeration e = groupHT.elements(); e.hasMoreElements(); ) {
                CollaborationGroupImpl g = (CollaborationGroupImpl)e.nextElement();
                out.println(g.getName() + ":" + g.getDisplayName());
            }
            out.close();

            //save the user file
            f = new File(_confDir, userFileName);            
            out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(f), "UTF-8")));
            for (Enumeration e = userHT.elements(); e.hasMoreElements(); ) {
                CollaborationPrincipalImpl u = (CollaborationPrincipalImpl)e.nextElement();
                String pass = getPassword(u.getName());
                if (pass == null) pass = "";
                String groupsString = "";
                CollaborationGroupImpl g[] = getGroups();
                for (int i = 0; i < g.length; i++) {
                    if (!g[i].hasMember(u)) continue; //see if not member
                    if (!groupsString.equals("")) groupsString += ",";
                    groupsString += g[i].getName();
                }
                out.println(u.getName() + ":" + pass + ":" + u.getDisplayName() + ":" + groupsString );
            }
            out.close();     
        } catch (Exception e) {
            System.err.println("[SampleRealm] Save failed: " + e.toString());
        }        
    }
    
    public boolean isMemberOfGroup(CollaborationPrincipal p, CollaborationGroup g) throws RealmException
    {
        return false; // todo
    }

    /**
     * get all CollaborationGroupImpls
     *
     * @return array of all CollaborationGroupImpls
     * @exception com.iplanet.im.server.RealmException
     */
    private CollaborationGroupImpl[] getGroups() throws RealmException {
        return _getCollaborationGroupImpls();
    }

    /**
     * get a list of all CollaborationGroupImpls
     * @return an array of CollaborationGroupImpls which may be length 0
     */
    private CollaborationGroupImpl [] _getCollaborationGroupImpls() {
        CollaborationGroupImpl [] ret = new CollaborationGroupImpl[groupHT.size()];
        Enumeration e = groupHT.elements();
        int i = 0;
        while (e.hasMoreElements()) {
            CollaborationGroupImpl g = (CollaborationGroupImpl)e.nextElement();
            ret[i++] = g;
        }
        return ret;
    }

    private CollaborationPrincipalImpl getUser(String id) {
        if (userHT == null) {
            resync();
        }
        return (CollaborationPrincipalImpl)userHT.get(id);
    }

    private String getPassword(String id) {
        if (passwordHT == null) {
            resync();
        }
        return (String)passwordHT.get(id);
    }

    private CollaborationGroupImpl getGroup(String id) {
        if (groupHT == null) {
            resync();
        }
        return (CollaborationGroupImpl)groupHT.get(id);
    }

    /**
     * set the users password
     * 
     * @param nlu server user to set it for
     * @param password the users new password
     */
    public void setPassword(CollaborationPrincipalImpl u, String password) {
        //mod = true;        
        passwordHT.put(u.getName(), password);
    }
    
    /**
     * get the current password for a user
     * 
     * @param nlu the user to get it from
     * @return the users password
     */
    public String getPassword(CollaborationPrincipalImpl u) {
        return (String)passwordHT.get(u.getName());
    }

    /*public String getSearchBase(CollaborationPrincipalImpl u) throws RealmException
    {
        return "";
    }*/
    public String getSearchBase(String domain) throws RealmException
    {
        return "";
    }        

    public CollaborationPrincipal createUser(String uid, String password, Map attributes) throws RealmException
    {
        String displayName = (String)attributes.get("cn");
        if (displayName == null) {
            displayName = (String)attributes.get("givenname") + " " + (String)attributes.get("sn");
        }
        CollaborationPrincipalImpl nlu = new CollaborationPrincipalImpl(uid, _serverDomain, displayName);

        passwordHT.put(nlu.getName(), password);
        userHT.put(nlu.getName(), nlu);

        saveOut();

        return nlu;
    }

    public void removeUser(String uid) throws RealmException
    {
	String name = StringUtility.getLocalPartFromAddress(uid);
	passwordHT.remove(name);
	userHT.remove(name);
    }

    public boolean changeUserPassword(String uid, String password) throws RealmException
    {
        return false;
    }

    public String getUserPassword(CollaborationPrincipal p) {
        return p.getProperty(PASSWORD_PROP);
    }




    /**
     * returns a handle to a user settings storage provider
     * @return UserSettingsStorageProvider object associated with this
     * Realm
     * @see UserSettingsStorageProvider
     */
    public UserSettingsStorageProvider getUserSettingsStorageProvider()
    {
        return new com.iplanet.im.server.FileUserSettings();
    }

    /**
     * returns a handle to a conference storage provider
     * @return ConferenceStorageProvider object associated with this
     * Realm
     * @see ConferenceStorageProvider
     */
    public ConferenceStorageProvider getConferenceStorageProvider()
    {
        // todo
        return new com.iplanet.im.server.FileGroupChatStorage();
    }

    /**
     * returns a handle to a conference storage provider
     * @return NewsStorageProvider object associated with this
     * Realm
     * @see NewsStorageProvider
     */
    public NewsStorageProvider getNewsStorageProvider()
    {
        // todo
        return new com.iplanet.im.server.FileNewsStorage();
    }

    public ConferenceStore getConferenceStore()
    {
        // todo
        return null;
    }
    //private static PolicyProvider _policyProvider = null;
    /**
     * returns a handle to a policy provider associated with this
     * Realm.
     * @return NewsStorageProvider object associated with this
     * Realm
     * @see NewsStorageProvider
     */
    public PolicyProvider getPolicyProvider()
    {
        return new SamplePolicyProvider();
    }

    public String getDomainName(String searchBase) throws RealmException {
        return "";
    }    

    /**
     * get the ignoredomains value
     */
    public boolean ignoreDomains() {
        return false;
    }
}








