/*
 * AbstractAttribute.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import org.jabberstudio.jso.StreamAttribute;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObjectVisitor;
import org.jabberstudio.jso.util.Utilities;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public abstract class AbstractAttribute extends AbstractObject implements StreamAttribute {
    //Fields
    private StreamElement       _Parent;

    /** Creates a new instance of AbstractAttribute */
    public AbstractAttribute(StreamElement p) {
        setParent(p);
    }
    
    //Methods
    /** <p>
     * Retrieves the parent <code>StreamElement</code> for this
     * <code>StreamNode</code>.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code>, if the
     * parent has not yet been defined.</p>
     *
     * @return The <code>StreamElement</code> parent.
     */
    public StreamElement getParent() {
        return _Parent;
    }
    /** <p>
     * Sets the parent <code>StreamElement</code> for this
     * <code>StreamNode</code>.</p>
     *
     * <p>
     * The value of <code>p</code> may be <code>null</code>, in which case
     * the current parent (if any)for this <code>StreamElement</code> is
     * detached.  If the parent cannot be changed, this method throws an
     * <code>IllegalStateException</code>.</p>
     *
     * @param p the <code>StreamElement</code> parent.
     * @throws IllegalStateException If the parent cannot be changed.
     */
    public void setParent(StreamElement p) throws IllegalStateException {
        _Parent = p;
    }
    
    public StreamDataFactory getDataFactory() {
        StreamElement   parent = getParent();
        
        return (parent != null) ? parent.getDataFactory() : null;
    }
    
    public String getLocalName() {
        return getNSI().getLocalName();
    }
    public String getNamespacePrefix() {
        StreamElement   parent = getParent();
        
        return (parent != null) ? parent.getNamespacePrefix(getNamespaceURI()) : null;
    }
    public String getNamespaceURI() {
        return getNSI().getNamespaceURI();
    }
    public String getQualifiedName() {
        StringBuffer        buffer = new StringBuffer();
        String              prefix = getNamespacePrefix();
        String              local = getLocalName();
        
        if (Utilities.isValidString(prefix))
            buffer.append(prefix).append(":");
        buffer.append(local);
        
        return buffer.toString();
    }

    public void accept(StreamObjectVisitor v) {
        v.visit(this);
    }
    
    public void detach() {
        StreamElement   parent = getParent();
        
        if (parent != null) {
            parent.setAttributeValue(getNSI(), null);
            setParent(null);
        }
    }
}