/*
 * AbstractNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.util.Locale;

import org.jabberstudio.jso.JSOImplementation;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNode;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$   
 * @since   JSO-0.2
 */
public abstract class AbstractNode extends AbstractObject implements StreamNode {
    //Fields
    private transient StreamDataFactory   _SDF;
    
    //Constructors
    public AbstractNode(StreamDataFactory sdf) {
        setDataFactory(sdf);
    }
    
    //Methods
    public StreamDataFactory getDataFactory() {
        StreamDataFactory   sdf;
        StreamElement       parent;
        
        if (_SDF != null) {
            sdf = _SDF;
        } else {
            parent = getParent();
            sdf = ((parent != null) ? parent.getDataFactory() : JSOImplementation.getInstance().getDataFactory());
        }
        
        return sdf;
    }
    public void setDataFactory(StreamDataFactory sdf) {
        _SDF = sdf;
    }
    
    public String getLanguage() {
        StreamElement   parent = getParent();
        
        return (parent != null) ? parent.getLanguage() : "";
    }
    public Locale getLocale() {
        StreamElement   parent = getParent();
        
        return ((parent != null) ? parent.getLocale() : Locale.getDefault());
    }
    
    public void detach() {
        StreamElement   parent = getParent();
        
        if (parent != null) {
            parent.remove(this);
        }
    }
}