/*
 * UnknownExtensionNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public class ExtensionNode extends ElementNode implements Extension {
    //Constructors
    /** Creates a new <code>ExtensionNode</code> */
    public ExtensionNode(StreamDataFactory sdf, NSI name) {
        super(sdf, name);
    }
    public ExtensionNode(StreamElement p, ExtensionNode base) {
        super(p, base);
    }
    
    //Methods
    /** <p>
     * Retrieves the namespace URI this <code>Extension</code> is
     * defined by.  The namespace is used to distiguish the type
     * of the <code>Extension</code>.</p>
     *
     * <p>
     * This method never returns <code>null</code> or "".</p>
     *
     * @return The namespace URI.
     */
    public String getNamespaceURI() {
        return getNSI().getNamespaceURI();
    }

    public StreamObject copy(StreamElement parent) {
        return new ExtensionNode(parent, this);
    }
    
    public void checkNSI(NSI check) throws IllegalArgumentException {
        super.checkNSI(check);
        if (!Utilities.equateStrings(getNamespaceURI(), check.getNamespaceURI()))
            throw new IllegalArgumentException("namespaces must be equal");
    }
    public void checkElement(StreamElement check) throws IllegalArgumentException {
    }
}