/*
 * XMLParseState.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.io;


import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.io.StreamParseState;
import org.jabberstudio.jso.util.Utilities;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public class XMLParseState implements StreamParseState {
    //Fields
    private XMLParseState   _Parent;
    private List            _Nodes = new java.util.LinkedList();

    private Type            _Type;
    private StreamElement   _Ctx;
    
    private NSI             _Name;
    private Map             _Nss = new java.util.TreeMap();
    private Map             _Attrs = new java.util.TreeMap(NSI.NS_LOOSE_COMPARATOR);
    
    private String          _Text;

    //Constructors
    public XMLParseState(StreamElement ctx, XMLParseState parent, Type type) {
        _Parent = parent;
        _Ctx = ctx;
        _Type = type;
    }
    
    //Methods
    public static XMLParseState createForElementStart(StreamElement ctx, XMLParseState p, NSI name, Map nss, Map attrs) {
        XMLParseState   sps = new XMLParseState(ctx, p, ELEMENT_START);
        
        sps.setName(name);
        sps.setNamespaces(nss);
        sps.setAttributes(attrs);
        
        return sps;
    }
    public static XMLParseState createForElementEnd(StreamElement ctx, XMLParseState p, NSI name) {
        XMLParseState   sps = new XMLParseState(ctx, p, ELEMENT_START);
        
        sps.setName(name);
        
        return sps;
    }
    public static XMLParseState createForText(StreamElement ctx, XMLParseState p, String txt) {
        XMLParseState   sps = new XMLParseState(ctx, p, TEXT);
        
        sps.setText(txt);
        
        return sps;
    }
    
    public XMLParseState getParent() {
        return _Parent;
    }
    public StreamElement getContext() {
        return _Ctx;
    }
    public StreamParseState.Type getType() {
        return _Type;
    }
    
    public int getDepth() {
        XMLParseState   parent = getParent();
        
        return ((parent != null) ? parent.getDepth() : 0) + 1;
    }
    
    public NSI getName() {
        return _Name;
    }
    private void setName(NSI name) {
        _Name = name;
    }
    
    public Map getNamespaces() {
        return _Nss;
    }
    private void setNamespaces(Map nss) {
        _Nss.clear();
        if (nss != null)
            _Nss.putAll(nss);
    }
    public Set getNamespaceURIs(int depth) {
        if ((depth < getDepth()) && (getParent() != null))
            return getParent().getNamespaceURIs(depth);
        return new java.util.TreeSet(getNamespaces().values());
    }
    public Set getNamespacePrefixes(int depth) {
        if ((depth < getDepth()) && (getParent() != null))
            return getParent().getNamespacePrefixes(depth);
        return getNamespaces().keySet();
    }
    public String getNamespacePrefix(String uri) {
        String      prefix = null;
        Iterator    itr = getNamespaces().values().iterator();
        
        if (uri == null)
            uri = "";
        
        while ((prefix == null) && itr.hasNext()) {
            String      temp = (String)itr.next();
            
            if (temp.equals(uri))
                prefix = temp;
        }
        
        if ((prefix == null) && (getParent() != null))
            prefix = getParent().getNamespacePrefix(uri);
        
        return prefix;
    }
    public String getNamespaceURI(String prefix) {
        String      uri;

        if (prefix == null)
            prefix = "";
        
        uri = (String)getNamespaces().get(prefix);
        if ((uri == null) && (getParent() != null))
            uri = getParent().getNamespaceURI(prefix);
        
        return uri;
    }

    public Map getAttributes() {
        return _Attrs;
    }
    private void setAttributes(Map attrs) {
        _Attrs.clear();
        if (attrs != null)
            _Attrs.putAll(attrs);
    }
    public Set getAttributeNames() {
        return getAttributes().keySet();
    }
    public String getAttributeValue(String name) throws IllegalArgumentException {
        return getAttributeValue(new NSI(name, null));
    }
    public String getAttributeValue(NSI name) throws IllegalArgumentException {
        if (name == null)
            throw new IllegalArgumentException("Name cannot be null");
        
        if (!Utilities.isValidString(name.getLocalName()))
            throw new IllegalArgumentException("Local Name cannot be null or \"\"");
        
        return (String)getAttributes().get(name);
    }
    
    public String getText() {
        return _Text;
    }
    private void setText(String txt) {
        _Text = ((txt != null) ? txt : "");
    }
    
    public String getXML() {
        return "";
    }
}