/*
 * Context.java
 *
 * Created on August 27, 2004, 5:24 PM
 */

package net.outer_planes.jso.ixp;


import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import com.jclark.xml.tok.ContentToken;
import com.jclark.xml.tok.Encoding;
import com.jclark.xml.tok.ExtensibleTokenException;
import com.jclark.xml.tok.PartialTokenException;
import com.jclark.xml.tok.StringConversionCache;
import com.jclark.xml.tok.Token;
import com.jclark.xml.tok.TokenException;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.io.StreamParseState;
import org.jabberstudio.jso.io.StreamSource;
import org.jabberstudio.jso.util.Utilities;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class Context {
    //Fields
    private StreamElement           _Elem;
    private NamespaceManager        _Namespaces = new NamespaceManager(null, null);
    private List                    _States = new java.util.LinkedList();
    private int                     _Depth = 0;
    private boolean                 _EOD;
    
    private Encoding                _Enc;
    private StringConversionCache   _SCC;
    private boolean                 _InCData;
    private StringBuffer            _Text = new StringBuffer();

    //Constructors
    public Context(StreamElement e) {
        _Elem = e;
    }
    
    //Methods
    public StreamElement getContextElement() {
        return _Elem;
    }
    
    public Encoding getEncoding() {
        return _Enc;
    }
    public StringConversionCache getStringConverter() {
        return _SCC;
    }
    public void configureEncoding(Encoding enc) {
        _Enc = enc;
        _SCC = new StringConversionCache(enc);
    }
    
    public boolean isInCData() {
        return _InCData;
    }
    
    public boolean hasStates() {
        return !_States.isEmpty();
    }
    public StreamParseState popState() {
        return (StreamParseState)(!_States.isEmpty() ? _States.remove(0) : null);
    }
    public void pushState(StreamParseState sps) {
        _States.add(sps);
    }
    
    public NamespaceManager getNamespaces() {
        return _Namespaces;
    }
    public NamespaceManager popNamespaces() throws IxpException {
        NamespaceManager    nsm = getNamespaces();

        if (nsm.getParent() == null) {
            throw new IxpException("unexpected end-of-document encountered");
        } else {
            _Namespaces = nsm.getParent();
        }

        return nsm;
    }
    public NamespaceManager pushNamespaces(Map nss) {
        NamespaceManager    nsm = getNamespaces();

        _Namespaces = new NamespaceManager(nsm, nss);

        return getNamespaces();
    }
    
    public int getElementDepth() {
        return _Depth;
    }
    protected int incrementElementDepth() {
        return ++_Depth;
    }
    protected int decrementElementDepth() {
        return --_Depth;
    }
    
    public void startElement(NSI name, Map nss, Map attrs) throws IxpException {
        IxpParseState       state;
        
        if (isEOD())
            throw new InvalidXMLException("cannot start new element after document end");
        
        pushTextState();
        
        state = new IxpParseState(getContextElement(), getNamespaces(), StreamParseState.ELEMENT_START);
        state.setName(name);
        state.setAttributes(attrs);
        pushState(state);
        
        incrementElementDepth();
    }
    public void endElement(NSI name) throws IxpException {
        IxpParseState       state;
        NamespaceManager    nsm = popNamespaces();
        
        if (isEOD())
            throw new InvalidXMLException("unexpected document end");
        
        pushTextState();
        
        state = new IxpParseState(getContextElement(), nsm, StreamParseState.ELEMENT_END);
        state.setName(name);
        pushState(state);

        if (decrementElementDepth() == 0) {
            setEOD(true);
        }
    }
    public void startCDATA() throws IxpException {
        _InCData = true;
    }
    public void endCDATA() throws IxpException {
        _InCData = false;
        
        pushTextState();
    }
    public void characters(String chars) throws IxpException {
        char            last = chars.charAt(chars.length() - 1);
        
        _Text.append(chars);
        if ((last == '\n') || (last == '\r'))
            pushTextState();
    }
    
    public boolean isEOD() {
        return _EOD;
    }
    public void setEOD(boolean eod) {
        _EOD = eod;
    }
    
    private void pushTextState() {
        IxpParseState   state;
        String          txt = _Text.toString();

        if (Utilities.isValidString(txt)) {
            _Text.setLength(0);

            state = new IxpParseState(getContextElement(), getNamespaces(), StreamParseState.TEXT);
            state.setText(txt);
            pushState(state);
        }
    }
    
    
    private Logger obtainLogger() {
        return Logger.getLogger(getClass());
    }
}