/*
 * IxpParseState.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.ixp;

import java.util.Map;
import java.util.Set;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.io.StreamParseState;
import org.jabberstudio.jso.util.Utilities;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.4
 */
public class IxpParseState implements StreamParseState {
    //Fields
    private StreamElement       _Ctx;
    
    private Type                _Type;
    
    private NSI                 _Name;
    private NamespaceManager    _NSS;
    private Map                 _Attrs = new java.util.TreeMap(NSI.NS_LOOSE_COMPARATOR);
    private String              _Text;
    
    //Constructors
    public IxpParseState(StreamElement ctx, NamespaceManager nss, Type type) {
        _Ctx = ctx;
        _NSS = nss;
        _Type = type;
    }
    
    //Methods -- Basics
    public StreamElement getContext() {
        return _Ctx;
    }
    
    public Type getType() {
        return _Type;
    }
    public void setType(Type t) throws IllegalArgumentException {
        if (t == null)
            throw new IllegalArgumentException("Type cannot be null");
        
        _Type = t;
    }

    public int getDepth() {
        return _NSS.getDepth();
    }

    public String getXML() {
        return null;
    }
    
    //Methods -- Element Name
    public NSI getName() {
        return _Name;
    }
    public void setName(NSI name) {
        _Name = name;
    }
    public void setName(String localname, String uri) {
        setName(new NSI(localname, uri));
    }
    
    //Methods -- Namespaces
    public Map getNamespaces() {
        return _NSS.getNamespaces();
    }
    public void setNamespaces(Map nss) {
        _NSS.setNamespaces(nss);
    }
    public Set getNamespaceURIs(int depth) {
        return _NSS.getNamespaceURIs(depth);
    }
    public Set getNamespacePrefixes(int depth) {
        return _NSS.getNamespacePrefixes(depth);
    }
    public String getNamespaceURI(String prefix) {
        return _NSS.getNamespaceURI(prefix);
    }
    public String getNamespacePrefix(String uri) {
        return _NSS.getNamespacePrefix(uri);
    }
    
    //Methods -- Attributes
    public Map getAttributes() {
        return _Attrs;
    }
    public void setAttributes(Map attrs) {
        _Attrs.clear();
        _Attrs.putAll(attrs);
    }
    public Set getAttributeNames() {
        return getAttributes().keySet();
    }
    public String getAttributeValue(String name) throws IllegalArgumentException {
        return getAttributeValue(new NSI(name, null));
    }
    public String getAttributeValue(NSI name) throws IllegalArgumentException {
        if (name == null)
            throw new IllegalArgumentException("Name cannot be null");
        
        if (!Utilities.isValidString(name.getLocalName()))
            throw new IllegalArgumentException("Local Name cannot be null or \"\"");
        
        return (String)getAttributes().get(name);
    }

    //Methods -- Text
    public String getText() {
        return _Text;
    }
    public void setText(String text) {
        _Text = text;
    }
    
    
    public String toString() {
        Type                type = getType();
        StringBuffer        buffer = new StringBuffer("parse state " + type);
        
        if          (getType() == ELEMENT_START) {
            buffer.append(" (");
            buffer.append("name=").append(getName());
            buffer.append(";nss=").append(getNamespaces());
            buffer.append(";attrs=").append(getAttributes());
            buffer.append(")");
        } else if   (getType() == ELEMENT_END) {
            buffer.append(" (");
            buffer.append("name=").append(getName());
            buffer.append(")");
        } else if   (getType() == TEXT) {
            buffer.append(" (").append(getText()).append(")");
        }
        
        return buffer.toString();
    }
}