/*
 * SASLFailurePacketNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.sasl;


import java.util.Map;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.sasl.SASLFailurePacket;
import org.jabberstudio.jso.util.Utilities;


/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class SASLFailurePacketNode extends SASLPacketNode implements SASLFailurePacket {
    //"Constants"
    public static final Map     CONDITIONS;
    
    static {
        Map     temp = new java.util.TreeMap();
        NSI     name;
        
        temp.put(ABORTED_CONDITION, new NSI(ABORTED_CONDITION, NAMESPACE));
        temp.put(INCORRECT_ENCODING_CONDITION, new NSI(INCORRECT_ENCODING_CONDITION, NAMESPACE));
        temp.put(INVALID_AUTHZID_CONDITION , new NSI(INVALID_AUTHZID_CONDITION, NAMESPACE));
        temp.put(INVALID_MECHANISM_CONDITION, new NSI(INVALID_MECHANISM_CONDITION, NAMESPACE));
        temp.put(MECHANISM_TOO_WEAK_CONDITION, new NSI(MECHANISM_TOO_WEAK_CONDITION, NAMESPACE));
        temp.put(NOT_AUTHORIZED_CONDITION , new NSI(NOT_AUTHORIZED_CONDITION , NAMESPACE));
        temp.put(TEMPORARY_AUTH_FAILURE_CONDITION, new NSI(TEMPORARY_AUTH_FAILURE_CONDITION, NAMESPACE));
        
        CONDITIONS = java.util.Collections.unmodifiableMap(temp);
    }
    
    //Constructors
    public SASLFailurePacketNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected SASLFailurePacketNode(StreamElement parent, SASLFailurePacketNode base) {
        super(parent, base);
    }

    //Methods
    public StreamObject copy(StreamElement parent) {
        return new SASLFailurePacketNode(parent, this);
    }
    
    public String getCondition() {
        StreamElement   cond = getFirstElement(null, NAMESPACE);
        
        return ((cond != null) ? cond.getLocalName() : "");
    }
    public void setCondition(String cond) throws IllegalArgumentException {
        NSI     name = null;
        
        if (Utilities.isValidString(cond) && ((name = (NSI)CONDITIONS.get(cond)) == null))
            throw new IllegalArgumentException("Condition name is not known");
        
        clearElements();
        if (name != null)
            addElement(name);
    }
}