/*
 * SASLFeatureNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.sasl;


import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.sasl.SASLMechanismsFeature;
import org.jabberstudio.jso.util.Utilities;

import net.outer_planes.jso.ElementNode;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JO-0.12
 */
public class SASLFeatureNode extends ElementNode implements SASLMechanismsFeature {
    //"Constants"
    public static final NSI ELEM_MECHANISM  = new NSI("mechanism", NAMESPACE);
    
    public static final Map NAMESPACES = Collections.singletonMap("sasl", NAMESPACE);
    
    //Constructors
    public SASLFeatureNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected SASLFeatureNode(StreamElement parent, SASLFeatureNode base) {
        super(parent, base);
    }
    
    //Methods
    public List getMechanismNames() {
        Iterator        itr = listElements(ELEM_MECHANISM).iterator();
        StreamElement   elem;
        String          name;
        List            mechs = new java.util.ArrayList();
        
        while (itr.hasNext()) {
            elem = (StreamElement)itr.next();
            name = elem.normalizeTrimText();
            
            if (Utilities.isValidString(name))
                mechs.add(name);
        }
        
        return mechs;
    }
    public void setMechanismNames(List names) {
        Iterator    itr = ((names != null) ? names : Collections.EMPTY_LIST).iterator();
        Object      val;
        String      name;
        
        clearMechanismNames();
        while (itr.hasNext()) {
            val = itr.next();
            name = ((val != null) ? val.toString() : null);
            
            if (Utilities.isValidString(name))
                addElement(ELEM_MECHANISM).addText(name);
        }
    }

    public void addMechanismName(String name) throws IllegalArgumentException {
        if (!Utilities.isValidString(name))
            throw new IllegalArgumentException("name cannot be null or \"\"");
        
        if (!getMechanismNames().contains(name)) {
            addElement(ELEM_MECHANISM).addText(name);
        }
    }
    public void removeMechanismName(String name) throws IllegalArgumentException {
        Iterator        itr;
        boolean         removed = false;
        StreamElement   elem;

        if (!Utilities.isValidString(name))
            throw new IllegalArgumentException("name cannot be null or \"\"");
        
        itr = selectAll("sasl:mechanism", NAMESPACES).iterator();
        while (!removed && itr.hasNext()) {
            elem = (StreamElement)itr.next();
            
            if (Utilities.equateStrings(name, elem.normalizeTrimText())) {
                elem.detach();
                removed = true;
            }
        }
    }
    public void clearMechanismNames() {
        clearElements(ELEM_MECHANISM);
    }
    
    public StreamObject copy(StreamElement parent) {
        return new SASLFeatureNode(parent, this);
    }
}