/*
 * ServerMechanism.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.sasl;


import org.jabberstudio.jso.sasl.SASLAuthPacket;
import org.jabberstudio.jso.sasl.SASLFailedException;
import org.jabberstudio.jso.sasl.SASLFailurePacket;
import org.jabberstudio.jso.sasl.SASLPacket;
import org.jabberstudio.jso.sasl.SASLServerInfo;
import org.jabberstudio.jso.util.Utilities;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public abstract class ServerMechanism extends AbstractMechanism {
    //Constructors
    public ServerMechanism(SASLServerInfo si) {
        super(si.getDataFactory(), si.getServer(), si.getCallbackHandler(), si.getProperties());
    }
    
    //Methods
    public SASLPacket evaluate(SASLPacket input) throws IllegalArgumentException, IllegalStateException, SASLFailedException {
        Status              current = getCurrentStatus();
        SASLPacket.Action   act = (input != null) ? input.getAction() : null;
        SASLPacket          output = null;
        SASLFailurePacket   failed;
        
        if (current == READY) {
            if      (act == SASLPacket.AUTHENTICATE)
                output = evaluateAuth((SASLAuthPacket)input);
            else if (act == SASLPacket.RESPONSE)
                output = evaluateResponse(input);
            else if (act == SASLPacket.ABORT)
                throw completeFailure(SASLFailurePacket.ABORTED_CONDITION);
            else
                throw completeFailure(SASLFailurePacket.TEMPORARY_AUTH_FAILURE_CONDITION);
        } else if   (current == AUTHENTICATING) {
            if      (act == SASLPacket.RESPONSE)
                output = evaluateResponse(input);
            else if (act == SASLPacket.ABORT)
                throw completeFailure(SASLFailurePacket.ABORTED_CONDITION);
            else
                throw completeFailure(SASLFailurePacket.TEMPORARY_AUTH_FAILURE_CONDITION);
        } else {
            throw new IllegalStateException("mechanism is not in a compatible state");
        }
        
        return output;
    }
    
    protected SASLPacket evaluateAuth(SASLAuthPacket auth) throws IllegalStateException, SASLFailedException {
        SASLPacket  resp;
        
        if (!Utilities.equateStrings(getMechanismName(), auth.getMechanismName())) {
            throw completeFailure(SASLFailurePacket.INVALID_MECHANISM_CONDITION);
        }
        
        resp = (SASLPacket)getDataFactory().createPacketNode(SASLPacket.NAME_RESPONSE, SASLPacket.class);
        
        return resp;
    }
    protected abstract SASLPacket evaluateResponse(SASLPacket resp) throws IllegalStateException, SASLFailedException;
    
    protected void completeSuccess(SASLPacket success) {
        setCurrentStatus(SUCCEEDED);
   }
    protected SASLFailedException completeFailure(String cond) {
        SASLFailurePacket   failed = (SASLFailurePacket)getDataFactory().createPacketNode(SASLFailurePacket.NAME, SASLFailurePacket.class);
        
        setCurrentStatus(FAILED);
        failed.setCondition(cond);
        
        return new SASLFailedException(failed);
    }
}