/*
 * DateTimeProfileWrapper.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */


package net.outer_planes.jso.util;


import java.text.ParseException;
import java.util.Date;

import org.jabberstudio.jso.format.DateTimeProfileFormat;
import org.jabberstudio.jso.util.Utilities;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class DateTimeProfileWrapper {
    //Fields
    private DateTimeProfileFormat   _Format;
    
    private Date                    _Wrapped;
    private String                  _Spec;

    //Constructors
    public DateTimeProfileWrapper(DateTimeProfileFormat fmt, Date date) throws IllegalArgumentException {
        if (fmt == null)
            throw new IllegalArgumentException("Format cannot be null");
        if (date == null)
            throw new IllegalArgumentException("Date cannot be null");

        _Wrapped = date;
        _Spec = fmt.format(date);
    }
    public DateTimeProfileWrapper(DateTimeProfileFormat fmt, String spec) throws IllegalArgumentException {
        if (fmt == null)
            throw new IllegalArgumentException("Format cannot be null");
        if (!Utilities.isValidString(spec))
            throw new IllegalArgumentException("Specification cannot be null");

        try {
            _Wrapped = fmt.parse(spec);
        } catch (ParseException pe) {
            throw new IllegalArgumentException("specification does not match profile");
        }
        _Spec = spec;
    }

    //Methods
    public final Date toDate() {
        return _Wrapped;
    }
    public final String toString() {
        return _Spec;
    }
    
    public static DateTimeProfileWrapper create(int profile, Date date) throws IllegalArgumentException {
        return new DateTimeProfileWrapper(DateTimeProfileFormat.getInstance(profile), date);
    }
    public static DateTimeProfileWrapper create(int profile, String spec) throws IllegalArgumentException {
        return new DateTimeProfileWrapper(DateTimeProfileFormat.getInstance(profile), spec);
    }
}