/*
 * CommandActionsNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.commands;


import java.util.Iterator;
import java.util.Set;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.x.commands.*;

import net.outer_planes.jso.ElementNode;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class CommandActionsNode extends ElementNode implements CommandActions {
    //"Constants"
    public static final NSI     ATTRNAME_EXECUTE    = new NSI("execute", null);
    
    //Constructors
    public CommandActionsNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected CommandActionsNode(StreamElement parent, CommandActionsNode base) {
        super(parent, base);
    }
    
    //Methods
    public StreamObject copy(StreamElement parent) {
        return new CommandActionsNode(parent, this);
    }
    
    public Set getAllowedActions() {
        Iterator    itr = listElements(null, CommandQuery.NAMESPACE).iterator();
        Set         acts = new java.util.TreeSet();
        
        while (itr.hasNext()) {
            acts.add(CommandQueryNode.toAction(((StreamElement)itr.next()).getLocalName()));
        }
        
        return acts;
    }
    public void setAllowedActions(Set acts) {
        Iterator        itr;
        
        if (acts == null)
            acts = java.util.Collections.EMPTY_SET;
        
        clearElements(null, CommandQuery.NAMESPACE);
        itr = acts.iterator();
        while (itr.hasNext()) {
            Object      val = itr.next();
            
            if (val != null)
                addElement(val.toString(), CommandQuery.NAMESPACE);
        }
    }
    
    public CommandQuery.Action getExecuteAction() {
        Object                  val = getAttributeObject(ATTRNAME_EXECUTE);
        CommandQuery.Action     act = null;
        
        if          (val instanceof CommandQuery.Action) {
            act = (CommandQuery.Action)val;
        } else if   (val != null) {
            if ((act = CommandQueryNode.toAction(val.toString())) != null)
                setExecuteAction(act);
        }
        
        return act;
    }
    public void setExecuteAction(CommandQuery.Action act) {
        setAttributeObject(ATTRNAME_EXECUTE, act);
    }
    
    protected StreamElement getActionElement(CommandQuery.Action act) throws IllegalArgumentException {
        if (act == null)
            throw new IllegalArgumentException("action cannot be null");
        
        return getFirstElement(act.toString(), CommandQuery.NAMESPACE);
    }
    public boolean isActionAllowed(CommandQuery.Action act) throws IllegalArgumentException {
        return (getActionElement(act) != null);
    }
    public void setActionAllowed(CommandQuery.Action act, boolean allow) throws IllegalArgumentException {
        StreamElement   elem = getActionElement(act);
        
        if      ((elem == null) && allow)
            addElement(act.toString(), CommandQuery.NAMESPACE);
        else if ((elem != null) && !allow)
            elem.detach();
    }
}