/*
 * CommandQueryNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.commands;


import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jaxen.JaxenException;
import org.jaxen.XPath;
import org.jaxen.SimpleVariableContext;

import org.saxpath.SAXPathException;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.commands.*;

import net.outer_planes.jso.ExtensionNode;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class CommandQueryNode extends ExtensionNode implements CommandQuery {
    //"Constants"
    public static final NSI     ATTRNAME_ACTION     = new NSI("action", null);
    public static final NSI     ATTRNAME_STATUS     = new NSI("status", null);
    public static final NSI     ATTRNAME_NODE       = new NSI("node", null);
    public static final NSI     ATTRNAME_SESSIONID  = new NSI("sessionid", null);
    
    //Fields
    private static XPath    _InfoPath;
    private static XPath    _PayloadPath;
    
    //Constructors
    public CommandQueryNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected CommandQueryNode(StreamElement parent, CommandQueryNode base) {
        super(parent, base);
    }
    
    //Methods
    protected XPath getNoteXPath() {
        if (_InfoPath == null) {
            try {
                Map     nss = new java.util.TreeMap();
                nss.put("cmd", NAMESPACE);
                
                _InfoPath = getDataFactory().createXPath("cmd:note[@type=$type]", nss);
                _InfoPath.setVariableContext(new SimpleVariableContext());
            } catch (SAXPathException saxpe) {
            }
        }
        
        return _InfoPath;
    }
    protected XPath getPayloadXPath() {
        if (_PayloadPath == null) {
            try {
                _PayloadPath = getDataFactory().createXPath("*[namespace-uri()!='" + NAMESPACE + "']");
            } catch (SAXPathException saxpe) {
            }
        }
        
        return _PayloadPath;
    }
    
    public StreamObject copy(StreamElement parent) {
        return new CommandQueryNode(parent, this);
    }
    
    public String getNode() {
        String  node = getAttributeValue(ATTRNAME_NODE);
        
        return ((node != null) ? node : "");
    }
    public void setNode(String node) {
        setAttributeValue(ATTRNAME_NODE, Utilities.isValidString(node) ? node : null);
    }
    
    public String getSessionID() {
        String  id = getAttributeValue(ATTRNAME_SESSIONID);
        
        return ((id != null) ? id : "");
    }
    public void setSessionID(String id) {
        setAttributeValue(ATTRNAME_SESSIONID, Utilities.isValidString(id) ? id : null);
    }
    
    public CommandQuery.Action getAction() {
        Object                  val = getAttributeObject(ATTRNAME_ACTION);
        String                  name;
        CommandQuery.Action     act = null;
        
        if          (val instanceof CommandQuery.Action) {
            act = (CommandQuery.Action)val;
        } else if   (val != null) {
            if ((act = toAction(val.toString())) != null)
                setAction(act);
        }
        
        return act;
    }
    public void setAction(CommandQuery.Action act) {
        setAttributeObject(ATTRNAME_ACTION, act);
    }
    
    public CommandQuery.Status getStatus() {
        Object              val = getAttributeObject(ATTRNAME_STATUS);
        String              name;
        CommandQuery.Status stat = null;
        
        if          (val instanceof CommandQuery.Status) {
            stat = (CommandQuery.Status)val;
        } else if   (val != null) {
            name = val.toString();
            
            if      (Utilities.equateStrings(CANCELED.toString(), name))
                stat = CANCELED;
            else if (Utilities.equateStrings(COMPLETED.toString(), name))
                stat = COMPLETED;
            else if (Utilities.equateStrings(EXECUTING.toString(), name))
                stat = EXECUTING;
            
            if (stat != null)
                setStatus(stat);
        }
        
        return stat;
    }
    public void setStatus(CommandQuery.Status stat) {
        setAttributeObject(ATTRNAME_STATUS, stat);
    }
    
    public CommandActions getActionInfo() {
        CommandActions      acts = (CommandActions)getFirstElement(CommandActions.class);
        
        if (acts == null) {
            acts = (CommandActions)addElement(CommandActions.NAME, CommandActions.class);
        }
        
        return acts;
    }
    
    public List listNotes() {
        return listElements(CommandNote.class);
    }
    public CommandNote addNote(CommandNote.Type type, String msg) throws IllegalArgumentException {
        CommandNoteNode info;
        
        if (type == null)
            throw new IllegalArgumentException("type cannot be null");
        if (!Utilities.isValidString(msg))
            throw new IllegalArgumentException("message cannot be null or \"\"");
        
        info = (CommandNoteNode)addElement(CommandNoteNode.NAME, CommandNoteNode.class);
        info.setType(type);
        info.setMessage(msg);
        
        return info;
    }
    public void clearNotes() {
        clearElements(CommandNote.class);
    }
    
    public List listNotes(CommandNote.Type type) {
        SimpleVariableContext   varCtx;
        XPath                   xpath;
        List                    infos = java.util.Collections.EMPTY_LIST;
        
        if (type != null) {
            try {
                xpath = getNoteXPath();
                varCtx = new SimpleVariableContext();

                varCtx.setVariableValue("type", type.toString());
                xpath.setVariableContext(varCtx);
                infos = xpath.selectNodes(this);
            } catch (JaxenException je) {
            }
        } else {
            infos = listNotes();
        }
        
        return infos;
    }
    public void clearNotes(CommandNote.Type type) {
        Iterator        itr;
        
        if (type != null) {
            itr = listNotes(type).iterator();
            
            while (itr.hasNext())
                ((StreamElement)itr.next()).detach();
        } else {
            clearNotes();
        }
    }
    
    public StreamElement getPayload() {
        StreamElement   payload = null;
        
        try {
            payload = (StreamElement)getPayloadXPath().selectSingleNode(this);
        } catch (JaxenException je) {
        }
        
        return payload;
    }
    public void setPayload(StreamElement payload) {
        clearPayload();
        add(payload);
    }
    
    public List listPayload() {
        List        payloads = java.util.Collections.EMPTY_LIST;
        
        try {
            payloads = getPayloadXPath().selectNodes(this);
        } catch (JaxenException je) {
        }
        
        return payloads;
    }
    public void clearPayload() {
        Iterator        itr = listPayload().iterator();
        
        while (itr.hasNext())
            ((StreamElement)itr.next()).detach();
    }
    
    public static CommandQuery.Action toAction(String name) {
        CommandQuery.Action     act = null;
        
        if      (Utilities.equateStrings(CANCEL.toString(), name))
            act = CANCEL;
        else if (Utilities.equateStrings(COMPLETE.toString(), name))
            act = COMPLETE;
        else if (Utilities.equateStrings(EXECUTE.toString(), name))
            act = EXECUTE;
        else if (Utilities.equateStrings(NEXT.toString(), name))
            act = NEXT;
        else if (Utilities.equateStrings(PREV.toString(), name))
            act = PREV;
        
        return act;
    }
}