/*
 * CoreExtensionFactory.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.core;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.core.RosterExtension;
import org.jabberstudio.jso.x.core.RosterQuery;

import net.outer_planes.jso.AbstractElementFactory;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.4
 */
public class CoreExtensionFactory extends AbstractElementFactory {
    //Constructors
    public CoreExtensionFactory() {
        super(NSI.STRICT_COMPARATOR);
    }
    
    //Methods
    protected void registerSupportedElements() {
        NSI                 name;
        
        putSupportedElement(new AuthFeatureNode(null));
        putSupportedElement(new AuthQueryNode(null));
        
        putSupportedElement(new BindQueryNode(null));
        
        putSupportedElement(new PrivacyQueryNode(null));
        putSupportedElement(new PrivacyListNode(null));
        putSupportedElement(new PrivacyItemNode(null));
        
        putSupportedElement(new RegisterQueryNode(null));
        
        name = new NSI("item", RosterQuery.NAMESPACE);
        putSupportedElement(RosterNode.createRosterQuery(null));
        putSupportedElement(name, new RosterNode.Item(null, name));
        
        name = new NSI("item", RosterExtension.NAMESPACE);
        putSupportedElement(RosterNode.createRosterExtension(null));
        putSupportedElement(name, new RosterNode.Item(null, name));
        
        putSupportedElement(new SessionQueryNode(null));
    }
    protected NSI adjustNSI(NSI name) {
        String          localname, uri;

        uri = name.getNamespaceURI();
        localname = name.getLocalName();
        if      (uri.startsWith("jabber:iq:") && !Utilities.isValidString(localname))
            name = new NSI("query", uri);
        else if (uri.startsWith("jabber:x:") && !Utilities.isValidString(localname))
            name = new NSI("x", uri);
        
        return name;
    }
}