/*
 * PrivacyQueryNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.core;


import java.util.Iterator;
import java.util.List;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.core.PrivacyList;
import org.jabberstudio.jso.x.core.PrivacyQuery;

import net.outer_planes.jso.ExtensionNode;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class PrivacyQueryNode extends ExtensionNode implements PrivacyQuery {
    //"Constants"
    public static final NSI     ELEMNAME_ACTIVE     = new NSI("active", NAMESPACE);
    public static final NSI     ELEMNAME_DEFAULT    = new NSI("default", NAMESPACE);
    
    public static final NSI     ATTRNAME_NAME       = new NSI("name", null);
    
    //Constructors
    public PrivacyQueryNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected PrivacyQueryNode(StreamElement parent, PrivacyQueryNode base) {
        super(parent, base);
    }

    //Methods
    public StreamElement getActiveElement() {
        return getFirstElement(ELEMNAME_ACTIVE);
    }
    public boolean hasActive() {
        return (getActiveElement() != null);
    }
    public String getActive() {
        StreamElement   act = getActiveElement();
        String          name = null;
        
        if (act != null)
            name = act.getAttributeValue(ATTRNAME_NAME);
        
        return ((name != null) ? name : "");
    }
    public void setActive(String name) {
        StreamElement   act = getActiveElement();
        
        if (act == null)
            act = addElement(ELEMNAME_ACTIVE);
        
        act.setAttributeValue(ATTRNAME_NAME, (name != null) ? name : "");
    }
    public void removeActive() {
        StreamElement   act = getActiveElement();
        
        if (act != null)
            act.detach();
    }

    public StreamElement getDefaultElement() {
        return getFirstElement(ELEMNAME_DEFAULT);
    }
    public boolean hasDefault() {
        return (getDefaultElement() != null);
    }
    public String getDefault() {
        StreamElement   def = getDefaultElement();
        String          name = null;
        
        if (def != null)
            name = def.getAttributeValue(ATTRNAME_NAME);
        
        return ((name != null) ? name : "");
    }
    public void setDefault(String name) {
        StreamElement   def = getDefaultElement();
        
        if (def == null)
            def = addElement(ELEMNAME_DEFAULT);
        
        def.setAttributeValue(ATTRNAME_NAME, (name != null) ? name : "");
    }
    public void removeDefault() {
        StreamElement   def = getDefaultElement();
        
        if (def != null)
            def.detach();
    }
    
    public List listPrivacyLists() {
        return listElements(PrivacyList.class);
    }
    public PrivacyList getPrivacyList(String name) throws IllegalArgumentException {
        Iterator    itr = listPrivacyLists().iterator();
        PrivacyList list = null;
        
        if (!Utilities.isValidString(name))
            throw new IllegalArgumentException("name cannot be null or \"\"");
        
        while ((list == null) && itr.hasNext()) {
            list = (PrivacyList)itr.next();
            
            if (!Utilities.equateStrings(name, list.getName()))
                list = null;
        }
        
        return list;
    }
    public PrivacyList addPrivacyList(String name) throws IllegalArgumentException {
        PrivacyList     list = getPrivacyList(name);
        
        if (list == null) {
            PrivacyListNode temp;
            
            list = temp = (PrivacyListNode)addElement(PrivacyListNode.NAME, PrivacyListNode.class);
            
            temp.setName(name);
        }
        
        return list;
    }
    public void clearPrivacyLists() {
        clearElements(PrivacyList.class);
    }
    
    public StreamObject copy(StreamElement parent) {
        return new PrivacyQueryNode(parent, this);
    }
}