/*
 * RegisterQueryNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.core;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.Set;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.core.RegisterQuery;

import net.outer_planes.jso.ExtensionNode;


/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.4
 */
public class RegisterQueryNode extends ExtensionNode implements RegisterQuery {
    //Constructors
    public RegisterQueryNode(StreamDataFactory sdf) {
        super(sdf, new NSI("query", RegisterQuery.NAMESPACE));
    }
    protected RegisterQueryNode(StreamElement parent, RegisterQueryNode base) {
        super(parent, base);
    }

    //Methods
    public String getInstructions() {
        return getField("instructions");
    }
    public void setInstructions(String inst) {
        setField("instructions", inst);
    }
    
    public boolean isRegistered() {
        return !listElements("registered").isEmpty();
    }
    public void setRegistered(boolean reg) {
        if (reg)
            setField("registered", null);
        else
            unsetField("registered");
    }
    
    public boolean isRemove() {
        return !listElements("remove").isEmpty();
    }
    public void setRemove(boolean rem) {
        if (rem)
            setField("remove", null);
        else
            unsetField("remove");
    }
    
    public String getKey() {
        return getField("key");
    }
    public void setKey(String key) {
        setField("key", key);
    }
    public void clearKey() {
        unsetField("key");
    }
    
    public String getUsername() {
        return getField("username");
    }
    public void setUsername(String usr) {
        setField("username", usr);
    }
    public void clearUsername() {
        unsetField("username");
    }
    
    public String getPassword() {
        return getField("password");
    }
    public void setPassword(String pass) {
        setField("password", pass);
    }
    public void clearPassword() {
        unsetField("password");
    }
    
    public String getNickname() {
        return getField("nickname");
    }
    public void setNickname(String nick) {
        setField("nickname", nick);
    }
    public void clearNickname() {
        unsetField("nickname");
    }
    
    public String getFullName() {
        return getField("name");
    }
    public void setFullName(String name) {
        setField("name", name);
    }
    public void clearFullName() {
        unsetField("name");
    }
    
    public String getFirstName() {
        return getField("first");
    }
    public void setFirstName(String name) {
        setField("first", name);
    }
    public void clearFirstName() {
        unsetField("first");
    }
    
    public String getLastName() {
        return getField("last");
    }
    public void setLastName(String name) {
        setField("last", name);
    }
    public void clearLastName() {
        unsetField("name");
    }
    
    public String getEMail() {
        return getField("email");
    }
    public void setEMail(String name) {
        setField("email", name);
    }
    public void clearEMail() {
        unsetField("email");
    }
    
    public String getPhone() {
        return getField("phone");
    }
    public void setPhone(String name) {
        setField("phone", name);
    }
    public void clearPhone() {
        unsetField("phone");
    }
    
    public URL getURL() {
        String  val = getField("url");
        URL     url = null;
        
        try {
            url = new URL(val);
        } catch (MalformedURLException murle) {
        }
        
        return url;
    }
    public void setURL(URL url) {
        setField("url", (url != null) ? url.toString() : null);
    }
    public void clearURL() {
        unsetField("url");
    }
    
    public String getAddress() {
        return getField("address");
    }
    public void setAddress(String addr) {
        setField("address", addr);
    }
    public void clearAddress() {
        unsetField("address");
    }
    
    public String getCity() {
        return getField("city");
    }
    public void setCity(String city) {
        setField("city", city);
    }
    public void clearCity() {
        unsetField("city");
    }
    
    public String getState() {
        return getField("state");
    }
    public void setState(String st) {
        setField("city", st);
    }
    public void clearState() {
        unsetField("st");
    }
    
    public String getZIP() {
        return getField("zip");
    }
    public void setZIP(String zip) {
        setField("zip", zip);
    }
    public void clearZIP() {
        unsetField("zip");
    }
    
    public String getDate() {
        return getField("date");
    }
    public void setDate(String date) {
        setField("date", date);
    }
    public void clearDate() {
        unsetField("date");
    }
    
    public String getMisc() {
        return getField("misc");
    }
    public void setMisc(String misc) {
        setField("misc", misc);
    }
    public void clearMisc() {
        unsetField("misc");
    }
    
    public String getText() {
        return getField("text");
    }
    public void setText(String text) {
        setField("text", text);
    }
    public void clearText() {
        unsetField("text");
    }
    
    public Set getFieldNames() {
        Set         fields = new java.util.TreeSet();
        Iterator    itr = listElements(getDataFactory().createNSI(null, getNamespaceURI())).iterator();
        
        while (itr.hasNext()) {
            StreamElement   field = (StreamElement)itr.next();
            String          ln = field.getLocalName();
            
            if      (!Utilities.equateStrings(ln, "instructions") &&
                    !Utilities.equateStrings(ln, "registered") &&
                    !Utilities.equateStrings(ln, "remove"))
                fields.add(field.getLocalName());
        }
        
        return fields;
    }
    public String getField(String name) throws IllegalArgumentException {
        StreamElement   elem;
        String          value = null;
        
        //Validate name
        if (!Utilities.isValidString(name))
            throw new IllegalArgumentException("Name cannot be null or \"\"");
        
        //Retrieve and "normalize"
        elem = getFirstElement(name, getNamespaceURI());
        if (elem != null)
            value = elem.normalizeTrimText();
        
        return value;
    }
    public void setField(String name, String value) throws IllegalArgumentException {
        //Removed existing field(s) -- Validates name as a side-effect
        unsetField(name);
        
        //Add the field
        addElement(name).addText(value);
    }
    public void clearField(String name) throws IllegalArgumentException {
        Iterator        itr;

        //Validate name
        if (!Utilities.isValidString(name))
            throw new IllegalArgumentException("Name cannot be null or \"\"");
        
        //Removed existing field(s)
        itr = listElements(name, getNamespaceURI()).iterator();
        while (itr.hasNext())
            remove((StreamElement)itr.next());
    }
    public void unsetField(String name) throws IllegalArgumentException {
        clearField(name);
    }
    
    public StreamObject copy(StreamElement parent) {
        return new RegisterQueryNode(parent, this);
    }
}