/*
 * FeatureNegNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.fneg;


import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jaxen.JaxenException;
import org.jaxen.SimpleVariableContext;
import org.jaxen.XPath;
import org.saxpath.SAXPathException;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.fneg.FeatureNegQuery;
import org.jabberstudio.jso.x.xdata.XDataField;
import org.jabberstudio.jso.x.xdata.XDataForm;

import net.outer_planes.jso.ExtensionNode;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class FeatureNegNode extends ExtensionNode implements FeatureNegQuery {
    //Fields
    private XPath       _FeatPath;
    
    private static final Map    NSS;
    
    static {
        Map     temp = new java.util.TreeMap();
        
        temp.put("xdata", XDataForm.NAMESPACE);
        temp.put("fneg", NAMESPACE);
        
        NSS = Collections.unmodifiableMap(temp);
    }
    
    //Constructors
    public FeatureNegNode(StreamDataFactory sdf) {
        this(sdf, NAME);
    }
    public FeatureNegNode(StreamDataFactory sdf, NSI name) {
        super(sdf, name);
    }
    protected FeatureNegNode(StreamElement parent, FeatureNegNode base) {
        super(parent, base);
    }
    
    //Methods
    protected XPath getFeatureXPath() {
        if (_FeatPath == null) {
            try {
                _FeatPath = getDataFactory().createXPath(".//xdata:field[@var=$feat]", NSS);
            } catch (SAXPathException saxpe) {
            }
        }
        
        return _FeatPath;
    }
    protected XDataField getFeatureField(String feat) throws IllegalArgumentException {
        XPath                   xpath = getFeatureXPath();
        SimpleVariableContext   varCtx = new SimpleVariableContext();
        XDataField              field = null;
        
        if (!Utilities.isValidString(feat))
            throw new IllegalArgumentException("Feature var cannot be null or \"\"");
        
        varCtx.setVariableValue("feat", feat);
        xpath.setVariableContext(varCtx);
        try {
            field = (XDataField)xpath.selectSingleNode(this);
        } catch (JaxenException je) {
        }
        
        return field;
    }
    protected XDataField obtainFeatureField(String feat) throws IllegalArgumentException {
        XDataField      field = getFeatureField(feat);
        XDataForm       form = getForm();
        
        if (field == null)
            field = form.addField(feat, XDataField.LIST_SINGLE);
        
        return field;
    }
    
    public XDataForm getForm() {
        XDataForm       form = (XDataForm)getFirstElement(XDataForm.class);
        
        if (form == null) {
            form = (XDataForm)addElement(XDataForm.NAME, XDataForm.class);
        }
        
        return form;
    }
    
    public List listFeatureVars() {
        List        feats = selectAll(".//xdata:field", NSS);
        List        vars = new java.util.ArrayList(feats.size());
        Iterator    itr = feats.iterator();
        XDataField  field;
        
        while (itr.hasNext()) {
            field = (XDataField)itr.next();
            
            vars.add(field.getVar());
        }
        
        return vars;
    }
    public boolean hasFeature(String feat) throws IllegalArgumentException {
        return (getFeatureField(feat) != null);
    }
    public void putFeature(String feat, List opts) throws IllegalArgumentException {
        XDataField  field = obtainFeatureField(feat);
        Iterator    itr;
        
        if (opts == null)
            opts = Collections.EMPTY_LIST;
        
        field.clearOptions();
        itr = opts.iterator();
        while (itr.hasNext()) {
            Object      val = itr.next();
            
            if (val != null)
                field.addOption(val.toString());
        }
    }
    public void removeFeature(String feat) throws IllegalArgumentException {
        XDataField  field = getFeatureField(feat);
        
        if (field != null)
            field.detach();
    }
    
    public List getSupportedOptions(String feat) throws IllegalArgumentException {
        XDataField          field = getFeatureField(feat);
        XDataField.Option   opt;
        List                opts, supported;
        Iterator            itr;
        
        if (field != null) {
            opts        = field.listOptions();
            supported   = new java.util.ArrayList(opts.size());
            itr         = opts.iterator();
            
            while (itr.hasNext()) {
                opt = (XDataField.Option)itr.next();
                
                supported.add(opt.getValue());
            }
        } else {
            supported = Collections.EMPTY_LIST;
        }
        
        return supported;
    }
    
    public boolean hasPreferredOption(String feat) throws IllegalArgumentException {
        XDataField  field = getFeatureField(feat);
        
        return ((field != null) ? (field.getValue() != null) : false);
    }
    public String getPreferredOption(String feat) throws IllegalArgumentException {
        XDataField  field = getFeatureField(feat);
        String      preferred = null;
        
        if (field != null) {
            preferred = field.getValue();
        }
        
        if (preferred == null)
            preferred = "";
        
        return preferred;
    }
    public void setPreferredOption(String feat, String opt) throws IllegalArgumentException {
        XDataField  field = getFeatureField(feat);
        
        if          (opt != null) {
            if (field == null)
                field = obtainFeatureField(feat);
            field.setValue(opt);
        } else if   (field != null) {
            field.setValue(null);
        }
    }
    
    public StreamObject copy(StreamElement parent) {
        return new FeatureNegNode(parent, this);
    }
}