/*
 * DelayExtensionNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.info;


import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.JIDFormatException;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.x.info.DelayedExtension;

import net.outer_planes.jso.ExtensionNode;

/**
 *
 * @author  linuxwolf
 */
public class DelayExtensionNode extends ExtensionNode implements DelayedExtension {
    //Inner-classes
    private static class StampWrapper {
        //Fields
        private Date        _Stamp;
        
        //Constructors
        public StampWrapper(Date stamp) {
            _Stamp = stamp;
        }
        
        //Methods
        public final Date getDate() {
            return _Stamp;
        }
        public final String toString() {
            return PATTERN.format(getDate());
        }
    }
    
    //"Constants"
    public static final SimpleDateFormat    PATTERN = new SimpleDateFormat("yyyyMMdd'T'HH:mm:ss");
    
    public static final NSI                 ATTRNAME_FROM = new NSI("from", null);
    public static final NSI                 ATTRNAME_STAMP = new NSI("stamp", null);
    
    //Constructors
    public DelayExtensionNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    public DelayExtensionNode(StreamElement parent, DelayExtensionNode base) {
        super(parent, base);
    }

    //Methods
    public JID getFrom() {
        Object      val = getAttributeObject(ATTRNAME_FROM);
        JID         from = null;
        
        if          (val instanceof JID) {
            from = (JID)val;
        } else if   (val != null) {
            try {
                from = new JID(val);
                setFrom(from);
            } catch (JIDFormatException jfe) {
            }
        }
        
        return from;
    }
    public void setFrom(JID from) {
        setAttributeObject(ATTRNAME_FROM, from);
    }
    
    public Date getStamp() {
        Object      val = getAttributeObject(ATTRNAME_STAMP);
        Date        stamp = null;
        
        if          (val instanceof StampWrapper) {
            stamp = ((StampWrapper)val).getDate();
        } else if   (val != null) {
            try {
                stamp = PATTERN.parse(val.toString());
                setStamp(stamp);
            } catch (ParseException pe) {
            }
        }
        
        return stamp;
    }
    public void setStamp(Date stamp) {
        StampWrapper    wrap = null;
        
        if (stamp != null)
            wrap = new StampWrapper(stamp);
        setAttributeObject(ATTRNAME_STAMP, wrap);
    }
    
    public StreamObject copy(StreamElement parent) {
        return new DelayExtensionNode(parent, this);
    }
}