/*
 * FTRangeNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.sift;


import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.x.sift.FileTransferProfile;

import net.outer_planes.jso.ElementNode;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class FTRangeNode extends ElementNode implements FileTransferProfile.Range {
    //"Constants"
    public static final NSI     ATTRNAME_OFFSET     = new NSI("offset", null);
    public static final NSI     ATTRNAME_LENGTH     = new NSI("length", null);

    //Constructors
    public FTRangeNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected FTRangeNode(StreamElement parent, FTRangeNode base) {
        super(parent, base);
    }

    //Methods
    public boolean hasOffset() {
        return (getAttributeObject(ATTRNAME_OFFSET) != null);
    }
    public long getOffset() {
        Object      val = getAttributeObject(ATTRNAME_OFFSET);
        long        offset = 0;
        
        if          (val instanceof Number) {
            offset = ((Number)val).longValue();
        } else if   (val != null) {
            try {
                offset = Long.parseLong(val.toString());
                setOffset(offset);
            } catch (NumberFormatException nfe) {
                obtainLogger().error("could not convert offset 'string' to a number", nfe);
            }
        }
        
        return offset;
    }
    public void setOffset(long off) {
        setAttributeObject(ATTRNAME_OFFSET, (off >= 0) ? new Long(off) : null);
    }
    
    public boolean hasLength() {
        return (getAttributeObject(ATTRNAME_LENGTH) != null);
    }
    public long getLength() {
        StreamElement   parent = getParent();
        Object          val = getAttributeObject(ATTRNAME_LENGTH);
        long            length = 0;
        
        if          (val instanceof Number) {
            length = ((Number)val).longValue();
        } else if   (val != null) {
            try {
                length = Long.parseLong(val.toString());
                setLength(length);
            } catch (NumberFormatException nfe) {
                obtainLogger().error("could not convert length 'string' to number", nfe);
            }
        } else {
            if ((parent != null) && (parent instanceof FileTransferProfile))
                length = ((FileTransferProfile)parent).getSize();
        }
        
        return length;
    }
    public void setLength(long len) {
        setAttributeObject(ATTRNAME_LENGTH, (len >= 0) ? new Long(len) : null);
    }

    public StreamObject copy(StreamElement parent) {
        return new FTRangeNode(parent, this);
    }
}