/*
 * OptionNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.xdata;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.xdata.XDataField;
import org.jabberstudio.jso.x.xdata.XDataForm;

import net.outer_planes.jso.ElementNode;

/**
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public class OptionNode extends ElementNode implements XDataField.Option {
    //"Constants"
    public static final NSI     NAME            = new NSI("option", XDataForm.NAMESPACE);
    
    //Constructors
    public OptionNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected OptionNode(StreamElement parent, OptionNode base) {
        super(parent, base);
    }
    
    //Methods
    public boolean hasLabel() {
        return Utilities.isValidString(getAttributeValue(FieldNode.ATTRNAME_LABEL));
    }
    public String getLabel() {
        String  val = getAttributeValue(FieldNode.ATTRNAME_LABEL);
        
        return (Utilities.isValidString(val) ? val : getValue());
    }
    public void setLabel(String label) {
        setAttributeValue(FieldNode.ATTRNAME_LABEL, Utilities.isValidString(label) ? label : null);
    }
    
    public String getValue() {
        StreamElement   elem = getFirstElement(FieldNode.ELEMNAME_VALUE);
        
        return ((elem != null) ? elem.normalizeTrimText() : "");
    }
    public void setValue(String val) {
        clearElements(FieldNode.ELEMNAME_VALUE);
        
        addElement(FieldNode.ELEMNAME_VALUE).addText(val);
    }
    
    public StreamObject copy(StreamElement parent) {
        return new OptionNode(parent, this);
    }
}