/*
 * StreamAttribute.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


/**
 * <p>
 * Represents an element attribute in XML.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public interface StreamAttribute extends StreamObject {
    /** <p>
     * Retrieves the value of this <code>StreamAttribute</code>, as a
     * string.</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>, but can
     * be "".</p>
     *
     * @return The string value of this <code>StreamAttribute</code>.
     * @since   JSO-0.2
     */
    public String getValue();
    /** <p>
     * Sets the value of this <code>StreamAttribute</code>, as a string.</p>
     *
     * <p>
     * The value of <code>val</code> can be <code>null</code> or "", in which
     * case it is considered to be "".  If the value of <code>val</code> is
     * not acceptable for this <code>StreamAttribute</code>, an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * @param val The new string value for this <code>StreamAttribute</code>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.2
     */
    public void setValue(String val) throws IllegalArgumentException;
}