/*
 * StreamError.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;

/**
 * <p>
 * Interface representing stream-level errors.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public interface StreamError extends Packet, ErrorElement {
    //"Constants"
    /**
     * Namespace URI for stream-level error conditions.
     *
     * @since   JSO-0.12
     */
    public static final String CONDITION_NAMESPACE                  = "urn:ietf:params:xml:ns:xmpp-streams";

    /**
     * Condition for indicating the received XML could not be processed. This
     * is a generic form of the more specific error conditions.
     *
     * @since   JSO-0.12
     * @see     #BAD_NAMESPACE_PREFIX_CONDITION
     * @see     #INVALID_XML_CONDITION
     * @see     #RESTRICTED_XML_CONDITION
     * @see     #UNSUPPORTED_ENCODING_CONDITION
     * @see     #XML_NOT_WELL_FORMED_CONDITION
     */
    public static final String BAD_FORMAT_CONDITION                 = "bad-format";
    /**
     * <p>
     * Condition for indicating a missing or invalid use of namespace prefixes.
     *
     * @since   JSO-0.12
     * @see     #BAD_FORMAT_CONDITION
     */
    public static final String BAD_NAMESPACE_PREFIX_CONDITION       = "bad-namespace-prefix";
    /**
     * Condition for indicating the existing stream now conflicts with another
     * stream.
     *
     * @since   JSO-0.12
     */
    public static final String CONFLICT_CONDITION                   = "conflict";
    /**
     * Condition for indicating the stream is closing due to inactivity.
     *
     * @since   JSO-0.12
     */
    public static final String CONNECTION_TIMEOUT_CONDITION         = "connection-timeout";
    /**
     * Condition for indicating the requested server is no longer available.
     *
     * @since   JSO-0.12
     */
    public static final String HOST_GONE_CONDITION                  = "host-gone";
    /**
     * Condition for indicating the requested server is not known.
     *
     * @since   JSO-0.12
     */
    public static final String HOST_UNKNOWN_CONDITION               = "host-unknown";
    /**
     * Condition for indicating a stanza sent without correct addressing (such
     * as missing the "from" or "to").
     *
     * @since   JSO-0.12
     */
    public static final String IMPROPER_ADDRESSING_CONDITION        = "improper-addressing";
    /**
     * Condition for indicating the server encountered some internal failure
     * regarding the current stream.
     *
     * @since   JSO-0.12
     */
    public static final String INTERNAL_SERVER_ERROR_CONDITION      = "internal-server-error";
    /**
     * Condition for indicating the "from" value for a stream or stanza is
     * not authorized or allowed.
     *
     * @since   JSO-0.12
     */
    public static final String INVALID_FROM_CONDITION               = "invalid-from";
    /**
     * Condition for indicating the stream identifier is not allowed or valid.
     *
     * @since   JSO-0.12
     */
    public static final String INVALID_ID_CONDITION                 = "invalid-id";
    /**
     * Condition for indicating a required namespace is not allowed or valid.
     *
     * @since   JSO-0.12
     */
    public static final String INVALID_NAMESPACE_CONDITION          = "invalid-namespace";
    /**
     * Condition for indicating received data was not validated XML.
     *
     * @since   JSO-0.12
     * @see     #BAD_FORMAT_CONDITION
     */
    public static final String INVALID_XML_CONDITION                = "invalid-xml";
    /**
     * Condition for indicating data was received when not authorized to do so.
     *
     * @since   JSO-0.12
     */
    public static final String NOT_AUTHORIZED_CONDITION             = "not-authorized";
    /**
     * Condition for indicating some internal policy was violated.
     *
     * @since   JSO-0.12
     */
    public static final String POLICY_VIOLATION_CONDITION           = "policy-violation";
    /**
     * Condition for indicating that a required remote connection could not be
     * established.
     *
     * @since   JSO-0.12
     */
    public static final String REMOTE_CONNECTION_FAILED_CONDITION   = "remote-connection-failed";
    /**
     * Condition for indicating there are not enough resources to to service
     * the stream.
     *
     * @since   JSO-0.12
     */
    public static final String RESOURCE_CONSTRAINT_CONDITION        = "resource-constraint";
    /**
     * Condition for indicating XML data was received which is not allowed on
     * a stream (such as processing instructions or external entities).
     *
     * @since   JSO-0.12
     */
    public static final String RESTRICTED_XML_CONDITION             = "restricted-xml";
    /**
     * Condition for indicating the request should be made to another host. The
     * CDATA of the related element contains the hostname or IP address to try.
     *
     * @since   JSO-0.12
     */
    public static final String SEE_OTHER_HOST_CONDITION             = "see-other-host";
    /**
     * Condition for indicating the stream is being shutdown.
     *
     * @since   JSO-0.12
     */
    public static final String SYSTEM_SHUTDOWN_CONDITION            = "system-shutdown";
    /**
     * Condition for indiciating an unknown problem occured.
     *
     * @since   JSO-0.12
     */
    public static final String UNDEFINED_CONDITION                  = "undefined-condition";
    /**
     * Condition for indicating a character encoding was received that is not
     * supported.
     *
     * @since   JSO-0.12
     * @see     #BAD_FORMAT_CONDITION
     */
    public static final String UNSUPPORTED_ENCODING_CONDITION       = "unsupported-encoding";
    /**
     * Condition for indicating a stanza was received that is not supported.
     *
     * @since   JSO-0.12
     */
    public static final String UNSUPPORTED_STANZA_TYPE_CONDITION    = "unsupported-stanza-type";
    /**
     * Condition for indicating the stream version (or lack thereof) is not
     * supported.
     *
     * @since   JSO-0.12
     */
    public static final String UNSUPPORTED_VERSION_CONDITION        = "unsupported-version";
    /**
     * Condition for indicating the data received was not well-formed XML.
     *
     * @since   JSO-0.12
     * @see     #BAD_FORMAT_CONDITION
     */
    public static final String XML_NOT_WELL_FORMED_CONDITION        = "xml-not-well-fomred";
}