/*
 * StreamObjectVisitor.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


/**
 * <p>
 * Implements the Visitor pattern for <code>StreamNode</code> objects.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.2
 */
public interface StreamObjectVisitor {
    //Methods
    /**
     * <p>
     * Visits a <code>StreamAttribute</code>.  This method is called by
     * implementations of <code>StreamAttribute</code> for specialized
     * processing.</p>
     *
     * <p>
     * The value of <code>attr</code> is assumed to never be <code>null</code>,
     * since this method is not called directly by "users".</p>
     *
     * @param attr The <code>StreamAttribute</code> being visited.
     * @since   JSO-0.2
     */
    public void visit(StreamAttribute attr);
    /**
     * <p>
     * Visits a <code>StreamNamespace</code>.  This method is called by
     * implementations of <code>StreamNamespace</code> for specialized
     * processing.</p>
     *
     * <p>
     * The value of <code>ns</code> is assumed to never be <code>null</code>,
     * since this method is not called directly by "users".</p>
     *
     * @param ns The <code>StreamNamespace</code> being visited.
     * @since   JSO-0.4.1
     */
    public void visit(StreamNamespace ns);
    /**
     * <p>
     * Visits a <code>StreamElement</code>.  This method is called by
     * implementations of <code>StreamElement</code> for specialized
     * processing.</p>
     *
     * <p>
     * The value of <code>elem</code> is assumed to never be <code>null</code>,
     * since this method is not called directly by "users".</p>
     *
     * @param elem The <code>StreamElement</code> being visited.
     * @since   JSO-0.2
     */
    public void visit(StreamElement elem);
    /**
     * <p>
     * Visits a <code>StreamText</code>.  This method is called by
     * implementations of <code>StreamText</code> for specialized
     * processing.</p>
     *
     * <p>
     * The value of <code>text</code> is assumed to never be <code>null</code>,
     * since this method is not called directly by "users".</p>
     *
     * @param text The <code>StreamText</code> being visited.
     * @since   JSO-0.2
     */
    public void visit(StreamText text);
    
    /**
     * <p>
     * Visits a <code>SerializedNodes</code>.  This method is called by
     * implementations of <code>SerializedNodes</code> for specialized
     * processing.</p>
     *
     * <p>
     * The value of <code>text</code> is assumed to never be <code>null</code>,
     * since this method is not called directly by "users".</p>
     *
     * @param text The <code>SerializedNodes</code> being visited.
     */
    public void visit(SerializedNodes text);
}