/*
 * XMLExporter.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.io;

import org.jabberstudio.jso.StreamElement;

/**
 * <p>
 * Interface to generate XML strings from <code>StreamElement</code>
 * objects.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.6
 */
public interface XMLExporter extends Exporter {
    //Methods
    /**
     * <p>
     * Generates an XML string from a <code>StreamElement</code>.</p>
     *
     * <p>
     * The export uses the <code>StreamContext</code> from {@link #getContext}
     * for any unknown namespace declarations.</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.</p>
     *
     * @param in The <code>StreamElement</code> to generate from.
     * @return The XML string for <code>in</code>
     * @throws RuntimeException If any errors occur generating the XML string.
     * @since   JSO-0.6
     */
    public String write(StreamElement in) throws RuntimeException;
}