/*
 * SASLMechanismsFeature.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.sasl;


import java.util.List;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamFeature;

/**
 * <p>
 * Interface for presenting the SASL stream feature.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public interface SASLMechanismsFeature extends StreamFeature {
    //"Constants"
    /** The namespace URI for <tt>SASLMechanismsFeature</tt>
     * (urn:ietf:params:xml:ns:xmpp-sasl).
     *
     * @since   JSO-0.12
     */
    public static final String  NAMESPACE   = "urn:ietf:params:xml:ns:xmpp-sasl";
    /** The qualified name for <tt>SASLMechanismsFeature</tt>
     * ({urn:ietf:params:xml:ns:xmpp-sasl}mechanisms).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME        = new NSI("mechanisms", NAMESPACE);
    
    //Methods
    /**
     * <p>
     * Retrieves the list of mechanism names reported for this
     * <tt>SASLMechanismFeature</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is a string. The order of the names in the returned
     * list is the preferred order of support, from most-secure to
     * least-secure.</p>
     *
     * @return The list of reported mechanism names.
     * @since   JSO-0.12
     */
    public List getMechanismNames();
    /**
     * <p>
     * Sets the list of mechanism names reported for this
     * <tt>SASLMechanismFeature</tt>. This method removes all existing
     * names before applying the given list of names.</p>
     *
     * <p>
     * The value of <tt>names</tt> may be <tt>null</tt>, in which case it is
     * considered to be an empty list. Each element in the provided list is
     * converted to a string (if not <tt>null</tt>).</p>
     *
     * @param names the new list of reported mechanism names.
     * @since   JSO-0.12
     */
    public void setMechanismNames(List names);
    /**
     * <p>
     * Adds the given mechanism name to this <tt>SASLMechanismFeature</tt>, if
     * not already present.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param name The mechanism name to add.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public void addMechanismName(String name) throws IllegalArgumentException;
    /**
     * <p>
     * Removes the given mechanism name from this
     * <tt>SASLMechanismFeature</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param name The mechanism name to remove.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public void removeMechanismName(String name) throws IllegalArgumentException;
    /**
     * <p>
     * Removes all the reported mechanism names from this
     * <tt>SASLMechanismFeature</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearMechanismNames();
}