/*
 * StartTLSPacket.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.tls;


import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.Packet;

/**
 * <p>
 * Primary interface for representing the packets for STARTTLS negotiation.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public interface StartTLSPacket extends Packet {
    //"Constants"
    /** The namespace URI for <tt>StartTLSPacket</tt>
     * (urn:ietf:params:xml:ns:xmpp-tls).
     *
     * @since   JSO-0.12
     */
    public static final String  NAMESPACE       = "urn:ietf:params:xml:ns:xmpp-tls";
    /** The qualified name for the "starttls" <tt>StartTLSPacket</tt>
     * ({urn:ietf:params:xml:ns:xmpp-tls}starttls).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME_STARTTLS   = new NSI("starttls", NAMESPACE);
    /** The qualified name for the "proceed" <tt>StartTLSPacket</tt>
     * ({urn:ietf:params:xml:ns:xmpp-tls}proceed).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME_PROCEED    = new NSI("proceed", NAMESPACE);
}