/*
 * NodeprepProfile.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */


package org.jabberstudio.jso.util.stringprep;

/**
 * <p>
 * Implements the nodeprep profile from XMPP-Core (Appendix A).</p>
 *
 * @see     <a href='http://www.ietf.org/rfc/rfc3920.txt'>Extensible Messaging and Presence Protocol (XMPP): Core</a>
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12.3
 */
public class NodeprepProfile extends StringprepProfile {
    //Fields
    private static  NodeprepProfile     _Inst;

    //Constructors
    /**
     * <p>
     * Creates a new <tt>NodeprepProfile</tt>.</p>
     *
     * @since   JSO-0.12.3
     */
    protected NodeprepProfile() {
        super(  true,
                true,
                new MappingTable[] {
                    MAPPING_B_1,
                    MAPPING_B_2
                },
                new ProhibitedTable[] {
                    PROHIBITED_C_1_1,
                    PROHIBITED_C_1_2,
                    PROHIBITED_C_2_1,
                    PROHIBITED_C_2_2,
                    PROHIBITED_C_3,
                    PROHIBITED_C_4,
                    PROHIBITED_C_5,
                    PROHIBITED_C_6,
                    PROHIBITED_C_7,
                    PROHIBITED_C_8,
                    ProhibitedTable.create(new char[][] { {'\"'}, {'&'}, {'\''}, {'/'}, {':'}, {'<'}, {'>'}, {'@'}})
                });
    }

    //Methods

    /**
     * <p>
     * Retrieves the global instance of <tt>NodeprepProfile</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return  The global <tt>NodeprepProfile</tt> instance.
     * @since   JSO-0.12.3
     */
    public static synchronized NodeprepProfile getInstance() {
        if (_Inst == null)
            _Inst = new NodeprepProfile();
        return _Inst;
    }
}