/*
 * ExpireExtension.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.info;


import java.util.Date;

import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.NSI;

/**
 * <p>
 * Interface for representing "jabber:x:expire" queries, as documented in
 * JEP-0023.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 * @see <a href="http://www.jabber.org/jeps/jep-0023.html">JEP-0023: Message Expiration</a>
 */
public interface ExpireExtension extends Extension {
    //"Constants"
    /**
     * The namespace URI for <tt>ExpireExtension</tt> (jabber:x:expire).
     *
     * @since   JSO-0.12
     */
    public static final String      NAMESPACE   = "jabber:x:expire";
    /**
     * The qualified name for <tt>ExpireExtension</tt> ({jabber:x:expire}x).
     *
     * @since   JSO-0.12
     */
    public static final NSI         NAME        = new NSI("x", NAMESPACE);
    
    //Methods
    /**
     * <p>
     * Retrieves the seconds for this <tt>ExpireExtension</tt>. This value is
     * the number of seconds the message is expected to be valid for.</p>
     *
     * <p>
     * The value returned by this method should never be less than or equal
     * to 0.</p>
     *
     * @return The number of seconds.
     * @since   JSO-0.12
     */
    public long getSeconds();
    /**
     * <p>
     * Sets the number of seconds for this <tt>ExpireExtension</tt>.
     *
     * <p>
     * If the value of <tt>secs</tt> may be less than 0, in which case it is
     * considered to be 0.</p>
     *
     * @param secs The new number of seconds.
     * @since   JSO-0.12
     */
    public void setSeconds(long secs);
    
    /**
     * <p>
     * Retrieves the stored time for this <tt>ExpireExtension</tt>. This value
     * is the time the containing message was stored, as a <tt>Date</tt>
     * object.</p>
     *
     * <p>
     * The value returned may be <tt>null</tt> if it has not been set.</p>
     *
     * @return The stored time.
     * @since   JSO-0.12
     */
    public Date getStored();
    /**
     * <p>
     * Sets the stored time for this <tt>ExpireExtension</tt>.
     *
     * <p>
     * If the value of <tt>stored</tt> is <tt>null</tt>, any previous value
     * is removed.</p>
     *
     * @param stored The new stored time.
     * @since   JSO-0.12
     */
    public void setStored(Date stored);
}