/*
 * VersionQuery.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.info;


import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.NSI;

/**
 * <p>
 * Interface for representing "jabber:iq:version" queries, as documented in
 * JEP-0092.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 * @see <a href="http://www.jabber.org/jeps/jep-0092.html">JEP-0092: Software Version</a>
 */
public interface VersionQuery extends Extension {
    //"Constants"
    /**
     * The namespace URI for <tt>VersionQuery</tt> (jabber:iq:version).
     *
     * @since   JSO-0.12
     */
    public static final String  NAMESPACE = "jabber:iq:version";
    /**
     * The qualified name for <tt>VersionQuery</tt> ({jabber:iq:version}query).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME = new NSI("query", NAMESPACE);
    
    //Methods
    /**
     * <p>
     * Retrieves the name for this <tt>VersionQuery</tt>. This value is usually
     * the name of the software application.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>, but may be ""
     * if it has not yet been set.</p>
     *
     * @return The name of the software.
     * @since   JSO-0.12
     */
    public String getName();
    /**
     * <p>
     * Sets the name for this <tt>VersionQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * @param name The name of the software.
     * @since   JSO-0.12
     */
    public void setName(String name);
    
    /**
     * <p>
     * Retrieves the operating-system for this <tt>VersionQuery</tt>. This value
     * is usually some form of human-readable identifier for the platform the
     * software is currently running on.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>, but may be ""
     * if it has not yet been set.</p>
     *
     * @return The operating-system of the software.
     * @since   JSO-0.12
     */
    public String getOS();
    /**
     * <p>
     * Sets the operating-system for this <tt>VersionQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>os</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * @param os The operating-system of the software.
     * @since   JSO-0.12
     */
    public void setOS(String os);

    /**
     * <p>
     * Retrieves the version for this <tt>VersionQuery</tt>. This value is
     * usually the version of the software application.</p>
     *
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>, but may be ""
     * if it has not yet been set.</p>
     *
     * @return The version of the software.
     * @since   JSO-0.12
     */
    public String getVersion();
    /**
     * <p>
     * Sets the version for this <tt>VersionQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>ver</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * @param ver The version of the software.
     * @since   JSO-0.12
     */
    public void setVersion(String ver);
}