/*
 * AbstractFunction.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.xpath;


import org.jaxen.Function;
import org.jaxen.SimpleFunctionContext;

/**
 * <p>
 * Abstract base class for all JSO-related XPath functions. It provides a
 * a method for retrieving the function's XPath name, and for registering
 * the function with a <tt>FunctionContext</tt>.</p>
 *
 * @author  linuxwolf
 * @version $VERSION$
 * @since   JSO-0.12
 */
public abstract class AbstractFunction implements Function {
    //Methods
    /**
     * <p>
     * Retrieves the "typical" function name for this
     * <tt>AbstractFunction</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt> or "".</p>
     *
     * @return The typical function name.
     * @since   JSO-0.12
     */
    public abstract String getFunctionName();
    
    /**
     * <p>
     * Registers this <tt>JidFunction</tt> to the given function context and
     * namespace URI. This method registers to the function {@link
     * #getFunctionName}.</p>
     *
     * <p>
     * The value of <tt>uri</tt> may be <tt>null</tt>, in which case it is
     * ignored.</p>
     *
     * <p>
     * The value of <tt>funcs</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param uri The namespace URI to register to.
     * @param funcs The function context to register to.
     * @throws IllegalArgumentException If any parameters are invalid.
     */
    public void register(String uri, SimpleFunctionContext funcs) throws IllegalArgumentException {
        if (funcs == null)
            throw new IllegalArgumentException("function context cannot be null");
        
        funcs.registerFunction(uri, getFunctionName(), this);
    }
}